// dllmain.cpp : Defines the entry point for the DLL application.
#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include "..\\Shared\\detours.h"

#pragma comment (lib, "..\\Shared\\detours.lib")
#pragma comment (lib, "..\\Shared\\detoured.lib")

// target pointer to un-instrumented API
static BOOL (WINAPI *RealDeleteFileA)(LPCSTR) = DeleteFileA;
//static UINT (WINAPI *RealWinExec)(LPCSTR, UINT) = WinExec;
static BOOL (WINAPI *RealCreateProcessW)(
			 LPCWSTR, LPWSTR,
             LPSECURITY_ATTRIBUTES,
             LPSECURITY_ATTRIBUTES,
             BOOL, DWORD, LPVOID, LPCWSTR,
             LPSTARTUPINFOW,
             LPPROCESS_INFORMATION) = CreateProcessW;

BOOL WINAPI HookCreateProcessW(LPCWSTR lpApplicationName,
     LPWSTR lpCommandLine,
     LPSECURITY_ATTRIBUTES lpProcessAttributes,
     LPSECURITY_ATTRIBUTES lpThreadAttributes,
     BOOL bInheritHandles,
     DWORD dwCreationFlags,
     LPVOID lpEnvironment,
     LPCWSTR lpCurrentDirectory,
     LPSTARTUPINFOW lpStartupInfo,
     LPPROCESS_INFORMATION lpProcessInformation
    )
{
	DWORD dwLastError = GetLastError();
	BOOL  bResult = FALSE;
	CHAR  szDetouredDll[MAX_PATH];
	CHAR  szDllName[MAX_PATH];
	HMODULE hMod1 = NULL, hMod2 = NULL;

	// get the full path to the detours DLL
	hMod1 = GetModuleHandleA("detoured.dll");
	GetModuleFileNameA(hMod1, szDetouredDll, MAX_PATH);

	// get the full path to the hooking DLL
	GetModuleHandleEx(
		GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, 
		(LPCTSTR)&HookCreateProcessW,
		&hMod2);

	GetModuleFileNameA(hMod2, szDllName, MAX_PATH);

	OutputDebugStringA(szDllName);
	OutputDebugStringA(szDetouredDll);
	OutputDebugStringA("\n");

	wprintf(L"[DetoursHooks] Intercepting the creation of %s\n", lpCommandLine);
	
	// route creation of new process through 
	// the detours API 
	bResult = DetourCreateProcessWithDll(
		lpApplicationName,
		lpCommandLine, 
		lpProcessAttributes,
		lpThreadAttributes,
		bInheritHandles,
		dwCreationFlags,
		lpEnvironment, 
		lpCurrentDirectory,
		lpStartupInfo, 
		lpProcessInformation, 
		szDetouredDll, 
		szDllName, 
		(PDETOUR_CREATE_PROCESS_ROUTINEW)RealCreateProcessW);

	SetLastError(dwLastError);
	return bResult;
}

// detours function 
BOOL WINAPI HookDeleteFileA(LPCSTR lpFileName)
{
	// save the last error
	DWORD dwLastError = GetLastError();

	// check if the parameter is valid
	if (lpFileName != NULL && strrchr(lpFileName, '\\') != NULL) 
	{
		// allocate memory for copied file name
		PCHAR lpNewFile = new CHAR[MAX_PATH*2];
		if (lpNewFile != NULL) 
		{
			// make sure you have a path called c:\archve
			sprintf_s(lpNewFile, 
				MAX_PATH, 
				"c:\\archive\\%s", 
				strrchr(lpFileName, '\\') + 1);
			// copy the file to be deleted into an archive 
			printf("[DetoursHooks] Copying %s => %s\n", lpFileName, lpNewFile);
			CopyFileA(lpFileName, lpNewFile, FALSE);
			delete[] lpNewFile;
		}
	}

	// restore last error
	SetLastError(dwLastError);
	return RealDeleteFileA(lpFileName);
}

extern "C" __declspec(dllexport) void DummyFunc(void)
{
	return;
}

BOOL APIENTRY DllMain(HMODULE hModule,
                       DWORD   dwReason,
                       LPVOID  lpReserved
					  )
{
	// install the hook(s)
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		DetourTransactionBegin();
        DetourUpdateThread(GetCurrentThread());
        DetourAttach(&(PVOID&)RealDeleteFileA, HookDeleteFileA);
		DetourAttach(&(PVOID&)RealCreateProcessW, HookCreateProcessW);
        DetourTransactionCommit();
	} 
	// uninstall the hook(s)
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		DetourTransactionBegin();
        DetourUpdateThread(GetCurrentThread());
        DetourDetach(&(PVOID&)RealDeleteFileA, HookDeleteFileA);
		DetourAttach(&(PVOID&)RealCreateProcessW, HookCreateProcessW);
        DetourTransactionCommit();
	}
	return TRUE;
}

